<!-- Bootstrap CSS -->
<link href="{{pathto('_static/css/bootstrap.min.css', 1)}}" rel="stylesheet">

<!-- Fonts -->
<link href="{{pathto('_static/css/font-awesome.min.css', 1)}}" rel="stylesheet">

<!-- Custom CSS -->
<link href="{{pathto('_static/css/combined.css', 1)}}" rel="stylesheet">

<!-- Search CSS -->
<link href="{{pathto('_static/css/search.css', 1)}}" rel="stylesheet">

<!-- Pygments CSS -->
<link href="{{pathto('_static/pygments.css', 1)}}" rel="stylesheet">
